using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace ConfigNS
{
    public class Config
    {
        /// <summary>
        /// Name of config-file.
        /// </summary>
        private string fileName;

        /// <summary>
        /// Path of config-file.
        /// </summary>
        private string filePath;

        /// <summary>
        /// Sections in the configuration.
        /// </summary>
        public Sections sections;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="fileName">Name of config-file.</param>
        /// <param name="filePath">Path of config-file.</param>
        public Config(string fileName, string filePath)
        {
            this.filePath = filePath;
            this.fileName = Path.Combine(this.filePath, fileName);
            this.sections = new Sections();
        }

        /// <summary>
        /// Constructor without file-path. Sets application's directory as file-path.
        /// </summary>
        /// <param name="fileName">Name of config-file.</param>
        public Config(string fileName)
            : this(fileName, Application.StartupPath)
        { }

        /// <summary>
        /// Constructor without parameters. Sets "config.xml" as filename
        /// and application's directory as filepath.
        /// </summary>
        public Config()
            : this("config.xml")
        { }

        /// <summary>
        /// Loads the config-file and reads all sections and settings.
        /// </summary>
        /// <returns>True. False if a setting couldn't be found in the file.</returns>
        public bool Load()
        {
            bool returnValue = true;
            XmlDocument xmldoc = new XmlDocument();

            // open the file
            try
            {
                xmldoc.Load(this.fileName);
            }
            catch (IOException ex1)
            {
                // create the file if it doesn't exist
                try
                {
                    this.Save();
                    xmldoc.Load(this.fileName);
                }
                catch (IOException ex3)
                {
                    throw new IOException("Die Konfigurationsdatei '" +
                        this.fileName + "' konnte nicht geladen und/oder erstellt werden: " + ex3.Message, ex1);
                }
            }
            catch (XmlException ex2)
            {
                throw new XmlException("Fehler beim Laden der Konfigurationsdatei '" +
                    this.fileName + "': " + ex2.Message, ex2);
            }

            // find sections
            foreach (Section sec in this.sections.Values)
            {
                // find settings
                foreach (Setting set in sec.settings.Values)
                {
                    XmlNode setNode = xmldoc.SelectSingleNode(
                        "/config/" + sec.name + "/" + set.name);
                    if ((setNode != null) && (setNode.InnerText.Length != 0))
                    {
                        // setting found
                        set.value = setNode.InnerText;
                        set.wasInFile = true;
                    }
                    else
                    {
                        // setting not found
                        set.value = set.defaultValue;
                        set.wasInFile = false;
                        returnValue = false;
                    }
                }
            }
            return returnValue;
        }

        /// <summary>
        /// Save all settings to the config-file.
        /// </summary>
        public void Save()
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?><config></config>");

            foreach (Section sec in this.sections.Values)
            {
                // add section
                XmlElement secElement = xmldoc.CreateElement(sec.name);
                xmldoc.DocumentElement.AppendChild(secElement);

                foreach (Setting set in sec.settings.Values)
                {
                    // add setting
                    XmlElement setElement = xmldoc.CreateElement(set.name);
                    setElement.InnerText = set.value;
                    secElement.AppendChild(setElement);
                }
            }

            // save config-file
            try
            {
                xmldoc.Save(this.fileName);
            }
            catch (IOException ex1)
            {
                throw new IOException("Fehler beim Speichern der Konfigurationsdatei '" +
                    this.fileName + "': " + ex1.Message);
            }
            catch (XmlException ex2)
            {
                throw new XmlException("Fehler beim Speichern der Konfigurationsdatei '" +
                    this.fileName + "': " + ex2.Message, ex2);
            }
        }

        /// <summary>
        /// Property FileName (readonly).
        /// </summary>
        public string FileName
        {
            get
            {
                return fileName;
            }
        }

        /// <summary>
        /// Property FilePath (readonly).
        /// </summary>
        public string FilePath
        {
            get
            {
                return filePath;
            }
        }

        /// <summary>
        /// Shows a form with which one can change the configuration.
        /// </summary>
        public void ShowConfigForm()
        {
            new ConfigForm(this).ShowDialog();
        }

        /// <summary>
        /// Overrides ToString().
        /// </summary>
        /// <returns>Full path and filename of config file.</returns>
        public override string ToString()
        {
            return this.FileName;
        }

        /// <summary>
        /// Reads configuration file and adds all sections and settings
        /// to the current object even if they haven't been defined before.
        /// All data is removed from current object before file is read.
        /// </summary>
        public void ReadFile()
        {
            XmlDocument xmldoc = new XmlDocument();

            // open the file
            try
            {
                xmldoc.Load(this.fileName);
            }
            catch (IOException ex1)
            {
                throw new IOException("Die Konfigurationsdatei '" +
                    this.fileName + "' konnte nicht geladen werden:\n" + ex1.Message, ex1);
            }
            catch (XmlException ex2)
            {
                throw new XmlException("XML-Fehler beim Laden der Konfigurationsdatei:\n" +
                    this.fileName + "\n" + ex2.Message, ex2);
            }

            // create sections and settings according to data in file
            this.sections = new Sections();
            foreach (XmlNode sec in xmldoc.SelectSingleNode("config").ChildNodes)
            {
                this.sections.Add(sec.Name);
                foreach (XmlNode set in xmldoc.SelectSingleNode("/config/" + sec.Name).ChildNodes)
                {
                    this.sections[sec.Name].settings.Add(set.Name, set.InnerText);
                }
            }
        }

        /// <summary>
        /// Checks whether all defined sections and settings were
        /// set in the config file.
        /// </summary>
        /// <returns>Bool if all sections/settings were set.</returns>
        public bool CheckConfig()
        {
            StringBuilder errorMessage = new StringBuilder();
            foreach (Section sec in this.sections.Values)
            {
                foreach (Setting set in sec.settings.Values)
                {
                    if (set.wasInFile == false)
                    {
                        errorMessage.Append(sec.name + "/" + set.name + "\n");
                    }
                }
            }
            if (errorMessage.Length != 0)
            {
                throw new Exception("Werte nicht definiert fr folgende Felder:\n" + errorMessage + "\nin Datei:\n" + this.FileName);
            }
            else
            {
                return true;
            }
        }
    }
}
